﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectAircraftCarrier.Drawnings;
using ProjectAircraftCarrier.MovementStrategy;

namespace ProjectAircraftCarrier;

/// <summary>
/// Форма работы с объектом "Авианосец"
/// </summary>
public partial class FormAircraftCarrier : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningWarship? _drawningWarship;

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    public DrawningWarship SetWarship
    {
        set
        {
            _drawningWarship = value;
            _drawningWarship.SetPictureSize(pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormAircraftCarrier()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки авианосца
    /// </summary>
    private void Draw()
    {
        if (_drawningWarship == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningWarship.DrawTransport(gr);
        pictureBoxAircraftCarrier.Image = bmp;
    }

    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningWarship == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningWarship.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningWarship.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningWarship.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningWarship.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningWarship == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if(_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableWarship(_drawningWarship), pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
        }

        if(_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if(_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}
